/*************************************************************************\
*                                                                         *
*    File        : rx_buffer.v                                            *
*    Description : rx buffer  Module                                      *
*    Designer    : Mahmud Galela (mgalela@vlsi.itb.ac.id)                 *
*                                                                         *
*    The function of this module is to hold packet data nibble to get     *
*    SFD detector ready to process the first nibble and do data           *
*    decapsulation by allowing packet data go through rx buffer           *                                                                         *
*    after the SFD detected.                                              *
*                                                                         *
\*************************************************************************/

module rx_buffer (
  clk, 
  reset_n, 
  rx_data, 
  data_tap, 
  data,
  data_en
  );
	
  input		clk, data_en, reset_n;
  input [3:0]	rx_data;
  output[3:0]	data_tap,data;
  
  reg   [3:0]   data_tap;
  reg   [3:0]   data_temp;
  
always @(posedge clk or negedge reset_n)
begin
  if (!reset_n) 
    data_tap <= 4'b0000;   
  else
    data_tap <= rx_data;
end

always @(posedge clk or negedge reset_n)
begin
  if (!reset_n) 
    data_temp <= 4'b0000;   
  else
    data_temp <= data_tap;
end

assign data = data_en ? data_temp : 4'bzzzz;

endmodule 

