/*************************************************************************\
*                                                                         *
*    File        : FCS_checker.v                                          *
*    Description : FCS Checker Module                                     *
*    Designer    : Mahmud Galela (mgalela@vlsi.itb.ac.id)                 *
*                                                                         *
*    The function of this module is to check the CRC code in FCS field.   *
*    The decapsulated frame is passed into FCS checker, and no error had  *                                                                 
*    occurred if the result number is the magic number 0xCo74DD7B.        *
\*************************************************************************/

`define BUFFER_INIT_VALUE   32'hFFFFFFFF
`define CRCO_RESET          9'b000000000
`define CRC_NOT		    9'b111111111

module fcs_check (
  clk,
  reset_n,
  FCS_start_check,
  data,
  crc_ok
  );

input             FCS_start_check;  
                                    
input  [3:0]      data;             
input             clk;              
input             reset_n;          
output            crc_ok;           
                                    
reg               crc_ok;           
reg [31:0]        crc_buffer;       

function [31:0]   count_crc;
input [3:0]       d;
input [31:0]      c;
reg [3:0]         d;
reg [31:0]        c;
reg [31:0]        new_crc;

begin
  new_crc[0]  = d[3] ^ c[28];
  new_crc[1]  = d[2] ^ d[3] ^ c[28] ^ c[29];
  new_crc[2]  = d[1] ^ d[2] ^ d[3] ^ c[28] ^ c[29] ^ c[30];
  new_crc[3]  = d[0] ^ d[1] ^ d[2] ^ c[29] ^ c[30] ^ c[31];
  new_crc[4]  = d[0] ^ d[1] ^ d[3] ^ c[0] ^ c[28] ^ c[30] ^ c[31];
  new_crc[5]  = d[0] ^ d[2] ^ d[3] ^ c[1] ^ c[28] ^ c[29] ^ c[31];
  new_crc[6]  = d[1] ^ d[2] ^ c[2] ^ c[29] ^ c[30];
  new_crc[7]  = d[0] ^ d[1] ^ d[3] ^ c[3] ^ c[28] ^ c[30] ^ c[31];
  new_crc[8]  = d[0] ^ d[2] ^ d[3] ^ c[4] ^ c[28] ^ c[29] ^ c[31];
  new_crc[9]  = d[1] ^ d[2] ^ c[5] ^ c[29] ^ c[30];
  new_crc[10] = d[0] ^ d[1] ^ d[3] ^ c[6] ^ c[28] ^ c[30] ^ c[31];
  new_crc[11] = d[0] ^ d[2] ^ d[3] ^ c[7] ^ c[28] ^ c[29] ^ c[31];
  new_crc[12] = d[1] ^ d[2] ^ d[3] ^ c[8] ^ c[28] ^ c[29] ^ c[30];
  new_crc[13] = d[0] ^ d[1] ^ d[2] ^ c[9] ^ c[29] ^ c[30] ^ c[31];
  new_crc[14] = d[0] ^ d[1] ^ c[10] ^ c[30] ^ c[31];
  new_crc[15] = d[0] ^ c[11] ^ c[31];
  new_crc[16] = d[3] ^ c[12] ^ c[28];
  new_crc[17] = d[2] ^ c[13] ^ c[29];
  new_crc[18] = d[1] ^ c[14] ^ c[30];
  new_crc[19] = d[0] ^ c[15] ^ c[31];
  new_crc[20] = c[16];
  new_crc[21] = c[17];
  new_crc[22] = d[3] ^ c[18] ^ c[28];
  new_crc[23] = d[2] ^ d[3] ^ c[19] ^ c[28] ^ c[29];
  new_crc[24] = d[1] ^ d[2] ^ c[20] ^ c[29] ^ c[30];
  new_crc[25] = d[0] ^ d[1] ^ c[21] ^ c[30] ^ c[31];
  new_crc[26] = d[0] ^ d[3] ^ c[22] ^ c[28] ^ c[31];
  new_crc[27] = d[2] ^ c[23] ^ c[29];
  new_crc[28] = d[1] ^ c[24] ^ c[30];
  new_crc[29] = d[0] ^ c[25] ^ c[31];
  new_crc[30] = c[26];
  new_crc[31] = c[27];
  count_crc = new_crc;
end
endfunction

always @(posedge clk)
      begin
        if (crc_buffer == 32'hc704dd7b) //:magic number
          crc_ok <= 1'b1;
        else
          crc_ok <= 1'b0;
      end

always @(posedge clk)
      begin
        if(!reset_n)
          crc_buffer <= `BUFFER_INIT_VALUE;
        else
        if(FCS_start_check)
          crc_buffer <= count_crc (data,crc_buffer);
        else
          crc_buffer <= `BUFFER_INIT_VALUE;
      end
        
endmodule
